/* 
   bton - tell tapisrv to turn on Bluetooth chip.

   Copyright (C) Ho Ming Shun      cyph
                                    at	
                               cyph.ath.cx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as
   published by the Free Software Foundation;

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
   IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
   CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
   ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

   ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
   COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
   SOFTWARE IS DISCLAIMED.
*/

/*
 * Apparently, ezx_bt tells tapisrv to enable the bluetooth chip when it starts up through
 * some kind of binary RPC protocol. tapisrv will use an AT command (AT+EBTP=1) to powerup 
 * bluetooth!! (what the hell does bluetooth got to do with the GSM module??)
 *
 * This program is created by running strace output through a Vim macro :)
 *
 * Based on observations of other users, it seems tapisrv returns the BD_ADDR to be programmed
 * into the bluetooth chip. bton will take this BD_ADDR and write it into /tmp/bd_addr
 * in the format that dload expects.
*/


#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>

#define END 0
#define WRITE 1
#define READ 2
#define MAX_DATA 12

struct rw_op {
	char op;
	unsigned char data[MAX_DATA];
	int len;

};

struct rw_op tapi_ops[] = {
	{ WRITE, {0x02, 0x81, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00}, 8 },
	{ WRITE, {0x38, 0x01, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00}, 12 },
	{ READ, {0x02, 0x81, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00}, 8 },
	{ READ, {0x00, 0x00, 0x00, 0x00}, 4 },
	{ WRITE, {0x0c, 0x86, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, 8 },
	{ READ, {0x0c, 0x86, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00}, 8 },
	{ READ, {0x00, 0x00, 0x00, 0x00}, 4 },
	{ WRITE, {0x0b, 0x86, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, 8 },
	{ READ, {0x0b, 0x86, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00}, 8 },
	{ READ, {0x00, 0x00, 0x00, 0x00}, 4 },
	{ WRITE, {0x0d, 0x86, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, 8 },
	{ READ, {0x0d, 0x86, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00}, 8 },
	//{ READ, {0x00, 0x00, 0x00, 0x00, 0x00, 0x12, 0x8a, 0x2d, 0x3d, 0x3e}, 10 },
	{ END },
};

void read_write(int fd, struct rw_op ops[]);
void read_bdaddr(int fd);
void dump(unsigned char *start, int len);

int main(int argc, char **argv) {
	int tapisock;
	struct sockaddr_un tapiaddr;
		
	if((tapisock = socket(PF_UNIX, SOCK_STREAM, 0)) < 0) {
		perror("cannot create socket");
		exit(1);
	}

	tapiaddr.sun_family = AF_UNIX;
	strcpy(tapiaddr.sun_path, "/tmp/tapisock");

	if(connect(tapisock, (struct sockaddr*)&tapiaddr, sizeof(tapiaddr.sun_family) + strlen(tapiaddr.sun_path))) {
		perror("cannot connect");
		exit(1);
	}

	printf("Connected. Starting read/write\n");
	
	read_write(tapisock, tapi_ops);
	read_bdaddr(tapisock);

	close(tapisock);

	return 0;
	
}

void read_bdaddr(int fd) {
	char buf[10];
	int readlen;
	FILE *bd_addr_file;

	readlen = 0;
	while(readlen != 10) {
		if((readlen += read(fd, buf + readlen, 10 - readlen)) < 0) {
			perror("read failed");
		}
	}

	printf("Read BDADDR : ");
	dump(buf, 10);
	printf("\n");

	bd_addr_file = fopen("/tmp/bd_addr", "w+");
	if(!bd_addr_file) {
		perror("error opening /tmp/bd_addr");
		exit(1);
	}

	fprintf(bd_addr_file, "%02x%02x%02x%02x%02x%02x\n", buf[4], buf[5], buf[6], buf[7], buf[8], buf[9]);

	fclose(bd_addr_file);


}

void read_write(int fd, struct rw_op ops[]) {
	struct rw_op *curr = ops;
	char buf[MAX_DATA];
	int readlen;

	while(curr->op != END) {
		switch(curr->op) {
			case WRITE:
				printf("Writing %02d bytes: ", curr->len);
				dump(curr->data, curr->len);
				printf("\n");

				if(write(fd, curr->data, curr->len) < 0) {
					perror("write failed");
					exit(1);
				}
				break;
			case READ:
				printf("Reading %02d bytes: \n", curr->len);

				readlen = 0;
				while(readlen != curr->len) {
					if((readlen += read(fd, buf + readlen, curr->len - readlen)) < 0) {
						perror("read failed");
					}
				}

				printf("Read      : ");
				dump(buf, curr->len);
				printf("\n");

				printf("Expecting : ");
				dump(curr->data, curr->len);
				printf("\n");

				if(memcmp(curr->data, buf, curr->len)) {
					printf("(wrong)\n");
				} else {
					printf("(correct)\n");
				}

				break;
		}
		curr++;
	}
}

void dump(unsigned char *start, int len) {
	int i;

	for(i = 0; i < len; i++) {
		printf("%02x ", (unsigned int)start[i]);
	}
}
