/* 
   ezx-hciattach - init bluetooth UART and attach HCI line discipline

   Copyright (C) Ho Ming Shun      cyph
                                    at	
                               cyph.ath.cx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as
   published by the Free Software Foundation;

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
   IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
   CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
   ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

   ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
   COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
   SOFTWARE IS DISCLAIMED.
*/

/*
 * Cobbled together from ezx_bt's strace output, all of its initialization ioctls it uses
 * to initialize the bluetooth UART interface is reproduced here faithfully (including some
 * private ioctls Motorola added to drivers/char/serial). After getting the UART to the exact 
 * settings that this fussy bluetooth chip requires, it attaches the bluetooth HCI UART H4
 * protocol line discipline (code lifted from original hciattach).
 */
#include <asm/termios.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>

#define HCIUARTSETPROTO _IOW('U', 200, int)
#define HCIUARTGETPROTO _IOR('U', 201, int)
#define N_HCI	15

#define HCI_UART_H4	0
#define HCI_UART_BCSP	1
#define HCI_UART_3WIRE	2
#define HCI_UART_H4DS	3

int main(int argc, char **argv) {
	int fd;
	int status;
	struct timespec sleeptime = {
		tv_sec: 0,
		tv_nsec: 10000000,
	};
	struct termios t;
	int i;

	fd = open("/dev/ttyS1", O_RDWR|O_NOCTTY);
	if(fd < 0) {
		perror("cannot open ttyS1");
		exit(1);
	}

	ioctl(fd, TCFLSH, 0x2);

	t.c_iflag=0x5;
	t.c_oflag=0;
	t.c_cflag=0x800018b4;
	t.c_lflag=0;
	t.c_line=0;
	memcpy(t.c_cc, "\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00", NCCS);
	ioctl(fd, TCSETS, &t);

	ioctl(fd, 0x5433, NULL);


	i = N_HCI;
	if (ioctl(fd, TIOCSETD, &i) < 0) {
		perror("Can't set line discipline");
		exit(1);
	}

	if (ioctl(fd, HCIUARTSETPROTO, HCI_UART_H4) < 0) {
		perror("Can't set device");
		exit(1);
	}

	if (argc > 1 && strcmp(argv[1], "-d") == 0) {
		if (fork()) {
			return 0;
		}
		for (i=0; i<20; i++)
			if (i != fd) close(i);
	}

	while (1) sleep(999999999);
	return 0;
}
